"""
Worker 监控模块

提供 Worker 列表、汇总统计、趋势等查询功能。
数据来源：Redis 实时数据 + queue_snapshot 表历史数据。
"""
from fastapi import APIRouter, HTTPException, Query, Path
from typing import Optional, Dict, Any
from datetime import datetime
import logging

from jettask.core.namespace import NamespaceContext
from jettask.webui.deps import NsDep
from jettask.webui.services.queue_service import QueueService
from jettask.webui.services.worker_service import WorkerService

router = APIRouter(prefix="/workers", tags=["Worker监控"])
logger = logging.getLogger(__name__)


async def _get_worker_service(ns: NamespaceContext) -> WorkerService:
    redis_client = await ns.get_redis_client(decode=False)
    return WorkerService(redis_client=redis_client, redis_prefix=ns.redis_prefix)



@router.get("/trend", summary="全局 Worker 数量趋势")
async def get_worker_trend(
    ns: NsDep,
    time_range: Optional[str] = Query("1h", description="时间范围 (1h, 6h, 24h, 7d)"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）")
) -> Dict[str, Any]:
    try:
        session = await ns.get_pg_session()
        async with session:
            return await QueueService.get_worker_trend(
                namespace=ns.name,
                pg_session=session,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range
            )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取全局 Worker 趋势失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))



@router.get("/summary", summary="全局 Worker 汇总统计")
async def get_worker_summary(ns: NsDep) -> Dict[str, Any]:
    try:
        service = await _get_worker_service(ns)
        workers = await service.get_all_workers()
        summary = await service.get_summary(workers)
        return {"success": True, "data": summary}
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取全局 Worker 汇总失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))



@router.get("/", summary="全局 Worker 列表")
async def get_worker_list(ns: NsDep) -> Dict[str, Any]:
    try:
        service = await _get_worker_service(ns)
        workers = await service.get_all_workers()
        return {"success": True, "data": workers, "total": len(workers)}
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取全局 Worker 列表失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))



@router.get("/{queue}/trend", summary="队列级 Worker 数量趋势")
async def get_worker_trend_by_queue(
    ns: NsDep,
    queue: str = Path(..., description="队列名称"),
    time_range: Optional[str] = Query("1h", description="时间范围 (1h, 6h, 24h, 7d)"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）")
) -> Dict[str, Any]:
    try:
        session = await ns.get_pg_session()
        async with session:
            return await QueueService.get_worker_trend_by_queue(
                namespace=ns.name,
                pg_session=session,
                queue=queue,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range
            )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取队列级 Worker 趋势失败: queue={queue}, error={e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))



@router.get("/{queue}/summary", summary="队列级 Worker 汇总统计")
async def get_worker_summary_by_queue(
    ns: NsDep,
    queue: str = Path(..., description="队列名称")
) -> Dict[str, Any]:
    try:
        service = await _get_worker_service(ns)
        workers = await service.get_workers_by_queue(queue)
        summary = await service.get_summary(workers)
        return {"success": True, "data": summary}
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取队列级 Worker 汇总失败: queue={queue}, error={e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))



@router.get("/{queue}", summary="队列级 Worker 列表")
async def get_worker_list_by_queue(
    ns: NsDep,
    queue: str = Path(..., description="队列名称")
) -> Dict[str, Any]:
    try:
        service = await _get_worker_service(ns)
        workers = await service.get_workers_by_queue(queue)
        return {"success": True, "data": workers, "total": len(workers)}
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取队列级 Worker 列表失败: queue={queue}, error={e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))



import os
import time
import asyncio
import logging
import json
from typing import Dict, List, Optional, Set, Callable, Any
import msgpack
from redis.asyncio.lock import Lock as AsyncLock
from jettask.db.pubsub import PubSubManager, PubSubSubscription

logger = logging.getLogger(__name__)



class WorkerManager:
    """Worker管理器 - Worker状态的唯一管理入口

    ⚠️ 重要：所有Worker状态的修改都必须通过这个类进行，不要直接操作Redis！

    职责:
    1. Worker ID 生成和复用
    2. Worker 注册表管理
    3. Worker 状态字段的读写
    4. Redis Pub/Sub 状态变更通知
    5. Worker 查询和统计
    """

    def __init__(self, redis_client, async_redis_client, redis_prefix: str = "jettask", event_pool=None,
                 queue_formatter=None, queue_registry=None, app=None, tasks: dict = None,
                 task_event_queues: dict = None, worker_id: str = None):
        self.sync_redis = redis_client
        self.redis = async_redis_client
        self.async_redis = async_redis_client
        self.redis_prefix = redis_prefix
        self.active_workers_key = f"{redis_prefix}:ACTIVE_WORKERS"
        self.workers_registry_key = f"{redis_prefix}:REGISTRY:WORKERS"
        self.event_pool = event_pool
        self.worker_prefix = 'WORKER'

        self.worker_state_channel = f"{redis_prefix}:WORKER_STATE_CHANGE"

        self._pubsub_subscription: Optional[PubSubSubscription] = None
        self._running = False
        self._callbacks: Set[Callable] = set()

        self._initialized = False
        self._last_full_sync = 0
        self._full_sync_interval = 60
        self._scan_counter = 0
        self._partial_check_interval = 10

        self._stop_recovery = False
        self._recovery_task = None  
        self.queue_formatter = queue_formatter or (lambda q: f"{self.redis_prefix}:QUEUE:{q}")
        self.worker_state_manager = app.worker_state_manager if (app and hasattr(app, 'worker_state_manager')) else None
        self.queue_registry = queue_registry
        self.app = app
        self.tasks = tasks or {}
        self.task_event_queues = task_event_queues or {}
        self.worker_id = worker_id

        self._stop_cleanup = False
        self._cleanup_task = None  

    def _get_worker_key(self, worker_id: str) -> str:
        return f"{self.redis_prefix}:WORKER:{worker_id}"

    async def initialize_worker(self, worker_id: str, worker_info: Dict[str, Any]):
        worker_key = self._get_worker_key(worker_id)
        current_time = time.time()

        worker_info.setdefault('is_alive', 'true')
        worker_info.setdefault('messages_transferred', 'false')
        worker_info.setdefault('created_at', str(current_time))
        worker_info.setdefault('last_heartbeat', str(current_time))

        pipeline = self.redis.pipeline()
        pipeline.hset(worker_key, mapping=worker_info)
        pipeline.zadd(self.active_workers_key, {worker_id: current_time})
        await pipeline.execute()

        logger.debug(f"Initialized worker {worker_id}")

    async def set_worker_online(self, worker_id: str, worker_data: dict = None):
        worker_key = self._get_worker_key(worker_id)
        old_alive = await self.redis.hget(worker_key, 'is_alive')
        old_alive = old_alive.decode('utf-8') if isinstance(old_alive, bytes) else old_alive

        current_time = time.time()
        pipeline = self.redis.pipeline()
        pipeline.hset(worker_key, 'is_alive', 'true')
        pipeline.hset(worker_key, 'last_heartbeat', str(current_time))

        if old_alive != 'true':
            pipeline.hset(worker_key, 'messages_transferred', 'false')

        if worker_data:
            pipeline.hset(worker_key, mapping=worker_data)

        pipeline.zadd(self.active_workers_key, {worker_id: current_time})
        await pipeline.execute()

        if old_alive != 'true':
            await self._publish_state_change(worker_id, 'online')
            logger.debug(f"Worker {worker_id} is now ONLINE")

    async def set_worker_offline(self, worker_id: str, reason: str = "unknown"):
        worker_key = self._get_worker_key(worker_id)
        old_alive = await self.redis.hget(worker_key, 'is_alive')
        old_alive = old_alive.decode('utf-8') if isinstance(old_alive, bytes) else old_alive

        current_time = time.time()
        pipeline = self.redis.pipeline()
        pipeline.hset(worker_key, 'messages_transferred', 'false')  
        pipeline.hset(worker_key, 'is_alive', 'false')
        pipeline.hset(worker_key, 'offline_reason', reason)
        pipeline.hset(worker_key, 'offline_time', str(current_time))
        pipeline.zrem(self.active_workers_key, worker_id)
        await pipeline.execute()

        if old_alive == 'true':
            await self._publish_state_change(worker_id, 'offline', reason)
            logger.debug(f"Worker {worker_id} is now OFFLINE (reason: {reason})")

    async def update_worker_heartbeat(self, worker_id: str, heartbeat_data: dict = None):
        worker_key = self._get_worker_key(worker_id)
        current_time = time.time()

        pipeline = self.redis.pipeline()
        pipeline.hset(worker_key, 'is_alive', 'true')
        pipeline.hset(worker_key, 'last_heartbeat', str(current_time))

        if heartbeat_data:
            pipeline.hset(worker_key, mapping=heartbeat_data)

        pipeline.zadd(self.active_workers_key, {worker_id: current_time})
        await pipeline.execute()

    async def update_worker_field(self, worker_id: str, field: str, value: str):
        worker_key = self._get_worker_key(worker_id)
        await self.redis.hset(worker_key, field, value)

    async def update_worker_fields(self, worker_id: str, fields: Dict[str, Any]):
        worker_key = self._get_worker_key(worker_id)
        await self.redis.hset(worker_key, mapping=fields)

    async def record_group_info(
        self,
        worker_id: str,
        queue: str,
        task_name: str,
        group_name: str,
        consumer_name: str,
        redis_prefix: str
    ):
        try:
            worker_key = self._get_worker_key(worker_id)

            group_info = {
                'queue': queue,
                'task_name': task_name,
                'group_name': group_name,
                'consumer_name': consumer_name,
                'stream_key': f"{redis_prefix}:QUEUE:{queue}"
            }

            field_name = f"group_info:{queue}"
            await self.redis.hset(
                worker_key,
                field_name,
                json.dumps(group_info)
            )

        except Exception as e:
            logger.error(f"Error recording group info for worker {worker_id}: {e}", exc_info=True)

    def update_worker_field_sync(self, worker_id: str, field: str, value: str):
        worker_key = self._get_worker_key(worker_id)
        self.sync_redis.hset(worker_key, field, value)
        logger.debug(f"Updated worker {worker_id} field {field}={value}")

    def update_worker_fields_sync(self, worker_id: str, fields: Dict[str, Any]):
        worker_key = self._get_worker_key(worker_id)
        self.sync_redis.hset(worker_key, mapping=fields)
        logger.debug(f"Updated worker {worker_id} fields: {list(fields.keys())}")

    async def increment_queue_stats(self, worker_id: str, queue: str,
                                   running_tasks_delta: int = None,
                                   success_count_increment: int = None,
                                   failed_count_increment: int = None,
                                   total_count_increment: int = None,
                                   processing_time_increment: float = None,
                                   latency_time_increment: float = None):
        worker_key = self._get_worker_key(worker_id)
        pipeline = self.redis.pipeline()

        if running_tasks_delta is not None and running_tasks_delta != 0:
            pipeline.hincrby(worker_key, f'{queue}:running_tasks', running_tasks_delta)

        if success_count_increment is not None:
            pipeline.hincrby(worker_key, f'{queue}:success_count', success_count_increment)

        if failed_count_increment is not None:
            pipeline.hincrby(worker_key, f'{queue}:failed_count', failed_count_increment)

        if total_count_increment is not None:
            pipeline.hincrby(worker_key, f'{queue}:total_count', total_count_increment)

        if processing_time_increment is not None:
            pipeline.hincrbyfloat(worker_key, f'{queue}:total_processing_time', processing_time_increment)

        if latency_time_increment is not None:
            pipeline.hincrbyfloat(worker_key, f'{queue}:total_latency_time', latency_time_increment)

        await pipeline.execute()

    async def get_queue_total_stats(self, worker_id: str, queue: str) -> dict:
        worker_key = self._get_worker_key(worker_id)
        fields = [
            f'{queue}:total_count',
            f'{queue}:total_processing_time',
            f'{queue}:total_latency_time'
        ]
        values = await self.redis.hmget(worker_key, fields)

        return {
            'total_count': int(values[0]) if values[0] else 0,
            'total_processing_time': float(values[1]) if values[1] else 0.0,
            'total_latency_time': float(values[2]) if values[2] else 0.0
        }

    async def update_queue_stats(self, worker_id: str, queue: str,
                                 running_tasks: int = None,
                                 avg_processing_time: float = None,
                                 avg_latency_time: float = None):
        worker_key = self._get_worker_key(worker_id)
        pipeline = self.redis.pipeline()

        if running_tasks is not None:
            pipeline.hset(worker_key, f'{queue}:running_tasks', str(running_tasks))

        if avg_processing_time is not None:
            pipeline.hset(worker_key, f'{queue}:avg_processing_time', f'{avg_processing_time:.3f}')

        if avg_latency_time is not None:
            pipeline.hset(worker_key, f'{queue}:avg_latency_time', f'{avg_latency_time:.3f}')

        await pipeline.execute()

    async def mark_messages_transferred(self, worker_id: str, transferred: bool = True):
        worker_key = self._get_worker_key(worker_id)
        await self.redis.hset(worker_key, 'messages_transferred', 'true' if transferred else 'false')

    async def get_worker_info(self, worker_id: str) -> Optional[Dict[str, str]]:
        worker_key = self._get_worker_key(worker_id)
        data = await self.redis.hgetall(worker_key)

        if not data:
            return None

        result = {}
        for k, v in data.items():
            key = k.decode('utf-8') if isinstance(k, bytes) else k
            value = v.decode('utf-8') if isinstance(v, bytes) else v
            result[key] = value

        return result

    async def get_worker_field(self, worker_id: str, field: str) -> Optional[str]:
        worker_key = self._get_worker_key(worker_id)
        value = await self.redis.hget(worker_key, field)

        if value is None:
            return None

        return value.decode('utf-8') if isinstance(value, bytes) else value

    async def is_worker_alive(self, worker_id: str) -> bool:
        is_alive = await self.get_worker_field(worker_id, 'is_alive')
        return is_alive == 'true'


    def generate_worker_id(self, prefix: str) -> str:
        import uuid
        return f"{prefix}-{uuid.uuid4().hex[:8]}-{os.getpid()}"

    async def find_reusable_worker_id(self, prefix: str) -> Optional[str]:
        try:
            offline_workers = await self.get_offline_workers()

            for worker_id in offline_workers:
                if isinstance(worker_id, bytes):
                    worker_id = worker_id.decode('utf-8')
                if worker_id.startswith(prefix):
                    logger.debug(f"Found reusable worker ID: {worker_id}")
                    return worker_id
        except Exception as e:
            logger.debug(f"Error finding reusable worker ID: {e}")

        return None


    async def register_worker(self, worker_id: str):
        await self.async_redis.sadd(self.workers_registry_key, worker_id)
        logger.debug(f"Registered worker: {worker_id}")

    async def unregister_worker(self, worker_id: str):
        await self.async_redis.srem(self.workers_registry_key, worker_id)
        logger.debug(f"Unregistered worker: {worker_id}")

    async def get_all_workers(self) -> Set[str]:
        return await self.async_redis.smembers(self.workers_registry_key)

    def get_all_workers_sync(self) -> Set[str]:
        return self.sync_redis.smembers(self.workers_registry_key)

    async def get_worker_count(self) -> int:
        return await self.async_redis.scard(self.workers_registry_key)

    async def get_offline_workers(self) -> Set[str]:
        all_workers = await self.get_all_workers()
        offline_workers = set()

        for worker_id in all_workers:
            if isinstance(worker_id, bytes):
                worker_id = worker_id.decode('utf-8')

            worker_key = f"{self.redis_prefix}:WORKER:{worker_id}"
            is_alive = await self.async_redis.hget(worker_key, 'is_alive')

            if is_alive:
                is_alive = is_alive.decode('utf-8') if isinstance(is_alive, bytes) else is_alive
                if is_alive != 'true':
                    offline_workers.add(worker_id)
            else:
                offline_workers.add(worker_id)

        return offline_workers

    async def get_workers_for_task(self, task_name: str, only_alive: bool = True) -> Set[str]:
        all_worker_ids = await self.get_all_workers()
        matched_workers = set()
        group_info_prefix = f"group_info:{self.redis_prefix}:QUEUE:"

        for worker_id in all_worker_ids:
            if isinstance(worker_id, bytes):
                worker_id = worker_id.decode('utf-8')

            worker_key = f"{self.redis_prefix}:WORKER:{worker_id}"
            worker_info = await self.async_redis.hgetall(worker_key)

            if not worker_info:
                continue

            decoded_info = {}
            for k, v in worker_info.items():
                key = k.decode('utf-8') if isinstance(k, bytes) else k
                val = v.decode('utf-8') if isinstance(v, bytes) else v
                decoded_info[key] = val

            if only_alive:
                is_alive = decoded_info.get('is_alive', 'false')
                if is_alive != 'true':
                    continue

            for key in decoded_info.keys():
                if key.startswith(group_info_prefix):
                    parts = key.split(':')
                    if len(parts) >= 5:
                        worker_task_name = parts[-1]  
                        if worker_task_name == task_name:
                            matched_workers.add(worker_id)
                            break

        return matched_workers

    async def get_active_worker_count_for_task(self, task_name: str) -> int:
        workers = await self.get_workers_for_task(task_name, only_alive=True)
        return len(workers)

    async def find_all_offline_workers(
        self,
        worker_prefix: str = 'WORKER'
    ):
        try:
            worker_ids = await self.get_all_workers()
            logger.debug(f"[Recovery] Scanning {len(worker_ids)} workers in registry")

            for worker_id in worker_ids:
                if isinstance(worker_id, bytes):
                    worker_id = worker_id.decode('utf-8')

                worker_key = f"{self.redis_prefix}:{worker_prefix}:{worker_id}"

                try:
                    decoded_worker_data = await self.get_worker_info(worker_id)

                    if not decoded_worker_data:
                        continue

                    is_alive = decoded_worker_data.get('is_alive', 'false') == 'true'
                    messages_transferred = decoded_worker_data.get('messages_transferred', 'false') == 'true'

                    if not is_alive and not messages_transferred:
                        logger.debug(
                            f"[Recovery] Found offline worker: {worker_id}, "
                            f"is_alive={is_alive}, "
                            f"messages_transferred={messages_transferred}"
                        )
                        yield (worker_key, decoded_worker_data)

                except Exception as e:
                    logger.error(f"[Recovery] Error processing worker key {worker_key}: {e}")
                    continue

        except Exception as e:
            logger.error(f"[Recovery] Error finding all offline workers: {e}")

    async def find_offline_workers_for_task(
        self,
        task_name: str,
        worker_prefix: str = 'WORKER'
    ) -> List[tuple]:
        all_offline_workers = await self.find_all_offline_workers(worker_prefix)
        logger.debug(f"[Recovery] Found {len(all_offline_workers)} offline workers, filtering for task {task_name}")

        task_offline_workers = []

        for worker_key, worker_data in all_offline_workers:
            has_task = False
            for key, value in worker_data.items():
                if key.startswith('group_info:'):
                    try:
                        group_info = json.loads(value)
                        if group_info.get('task_name') == task_name:
                            has_task = True
                            break
                    except Exception as e:
                        logger.error(f"[Recovery] Error parsing group_info: {e}")

            if has_task:
                worker_id = worker_key.split(':')[-1]
                logger.debug(
                    f"[Recovery] Found offline worker needing recovery: {worker_id}, task={task_name}"
                )
                task_offline_workers.append((worker_key, worker_data))
            else:
                worker_id = worker_key.split(':')[-1]
                logger.debug(
                    f"[Recovery] Worker {worker_id} is offline but not responsible for task {task_name}"
                )

        logger.debug(f"[Recovery] Found {len(task_offline_workers)} offline workers for task {task_name}")
        return task_offline_workers

    async def get_all_workers_info(self, only_alive: bool = True) -> Dict[str, Dict[str, str]]:
        pattern = f"{self.redis_prefix}:WORKER:*"
        result = {}

        cursor = 0
        while True:
            cursor, keys = await self.redis.scan(cursor, match=pattern, count=100)

            for key in keys:
                if isinstance(key, bytes):
                    key = key.decode('utf-8')

                parts = key.split(":")
                if len(parts) >= 3:
                    worker_id = parts[2]
                    worker_info = await self.get_worker_info(worker_id)
                    if worker_info:
                        if only_alive and worker_info.get('is_alive') != 'true':
                            continue
                        result[worker_id] = worker_info

            if cursor == 0:
                break

        return result

    async def delete_worker(self, worker_id: str, unregister: bool = True):
        worker_key = self._get_worker_key(worker_id)
        pipeline = self.redis.pipeline()
        pipeline.delete(worker_key)  
        pipeline.zrem(self.active_workers_key, worker_id)  

        if unregister:
            pipeline.srem(self.workers_registry_key, worker_id)  

        await pipeline.execute()
        logger.debug(f"Deleted worker {worker_id}" + (" and unregistered" if unregister else ""))

    async def _publish_state_change(self, worker_id: str, state: str, reason: str = None):
        message = {
            'worker_id': worker_id,
            'state': state,
            'timestamp': asyncio.get_event_loop().time()
        }

        if reason:
            message['reason'] = reason

        await self.redis.publish(
            self.worker_state_channel,
            json.dumps(message)
        )

        logger.debug(f"Published state change: {message}")

    async def start_listener(self):
        if self._running:
            logger.debug("Worker state listener already running")
            return

        self._running = True

        self._pubsub_subscription = await PubSubManager.subscribe(
            redis_client=self.redis,
            channel=self.worker_state_channel,
            on_message=self.handle_state_change,
            name="WorkerStateListener",
        )

        logger.debug(f"Started worker state listener on channel: {self.worker_state_channel}")

    async def stop_listener(self):
        if not self._running:
            return

        self._running = False

        if self._pubsub_subscription:
            await self._pubsub_subscription.stop()
            self._pubsub_subscription = None

        logger.debug("Stopped worker state listener")

    async def handle_state_change(self, data: dict):
        try:
            if data.get('state') == 'offline':
                worker_id = data.get('worker_id')
                if worker_id:
                    logger.debug(f"[StateManager] Worker {worker_id} offline event received, triggering recovery")
                    try:
                        await self._process_offline_workers_once()
                    except Exception as e:
                        logger.error(f"[StateManager] Error during offline worker recovery: {e}", exc_info=True)

            for callback in self._callbacks:
                try:
                    if asyncio.iscoroutinefunction(callback):
                        await callback(data)
                    else:
                        callback(data)
                except Exception as e:
                    logger.error(f"Error in state change callback: {e}")

        except Exception as e:
            logger.error(f"Error processing state change message: {e}")

    _handle_state_change_message = handle_state_change

    def register_callback(self, callback: Callable):
        self._callbacks.add(callback)
        logger.debug(f"Registered state change callback: {callback.__name__}")

    def unregister_callback(self, callback: Callable):
        self._callbacks.discard(callback)
        logger.debug(f"Unregistered state change callback: {callback.__name__}")


    async def scan_timeout_workers(self) -> List[Dict]:
        self._scan_counter += 1
        if self._scan_counter >= self._partial_check_interval:
            self._scan_counter = 0
            asyncio.create_task(self._partial_check())

        current_time = time.time()

        potential_timeout_worker_ids = await self.async_redis.zrangebyscore(
            self.active_workers_key,
            min=0,
            max=current_time - 1
        )

        if not potential_timeout_worker_ids:
            return []

        all_workers_info = await self.get_all_workers_info(only_alive=False)
        workers_data = [all_workers_info.get(wid) for wid in potential_timeout_worker_ids]

        result = []
        cleanup_pipeline = self.async_redis.pipeline()
        need_cleanup = False

        for worker_id, worker_data in zip(potential_timeout_worker_ids, workers_data):
            if not worker_data:
                cleanup_pipeline.zrem(self.active_workers_key, worker_id)
                cleanup_pipeline.srem(self.workers_registry_key, worker_id)
                need_cleanup = True
                continue

            worker_heartbeat_timeout = float(worker_data.get('heartbeat_timeout', 15.0))
            last_heartbeat = float(worker_data.get('last_heartbeat', 0))
            worker_cutoff_time = current_time - worker_heartbeat_timeout

            if last_heartbeat >= worker_cutoff_time:
                cleanup_pipeline.zadd(self.active_workers_key, {worker_id: last_heartbeat})
                need_cleanup = True
                continue

            is_alive = worker_data.get('is_alive', 'true') == 'true'
            if not is_alive:
                cleanup_pipeline.zrem(self.active_workers_key, worker_id)
                need_cleanup = True
                continue

            logger.debug(f"Worker {worker_id} timeout: last_heartbeat={last_heartbeat}, timeout={worker_heartbeat_timeout}s")
            worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"
            result.append({
                'worker_key': worker_key,
                'worker_data': worker_data,
                'worker_id': worker_id
            })

        if need_cleanup:
            await cleanup_pipeline.execute()

        if result:
            logger.debug(f"Found {len(result)} timeout workers")

        return result

    async def update_heartbeat(self, worker_id: str, heartbeat_time: Optional[float] = None):
        if heartbeat_time is None:
            heartbeat_time = time.time()

        pipeline = self.async_redis.pipeline()
        worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"

        pipeline.hset(worker_key, 'last_heartbeat', str(heartbeat_time))
        pipeline.zadd(self.active_workers_key, {worker_id: heartbeat_time})

        await pipeline.execute()

    async def add_worker(self, worker_id: str, worker_data: Dict):
        heartbeat_time = float(worker_data.get('last_heartbeat', time.time()))

        pipeline = self.async_redis.pipeline()
        worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"

        pipeline.hset(worker_key, mapping=worker_data)
        pipeline.zadd(self.active_workers_key, {worker_id: heartbeat_time})

        await pipeline.execute()
        logger.debug(f"Added worker {worker_id} to system")

    async def remove_worker(self, worker_id: str):
        await self.set_worker_offline(worker_id, reason="heartbeat_timeout")
        await self.async_redis.zrem(self.active_workers_key, worker_id)

    async def cleanup_stale_workers(self, max_age_seconds: float = 3600):
        current_time = time.time()
        cutoff_time = current_time - max_age_seconds

        stale_worker_ids = await self.async_redis.zrangebyscore(
            self.active_workers_key,
            min=0,
            max=cutoff_time
        )

        if not stale_worker_ids:
            return 0

        pipeline = self.async_redis.pipeline()

        for worker_id in stale_worker_ids:
            worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"
            pipeline.delete(worker_key)

        pipeline.zrem(self.active_workers_key, *stale_worker_ids)

        await pipeline.execute()

        logger.debug(f"Cleaned up {len(stale_worker_ids)} stale worker records")
        return len(stale_worker_ids)

    async def _partial_check(self):
        try:
            sample_size = min(10, await self.async_redis.zcard(self.active_workers_key))
            if sample_size == 0:
                return

            random_workers = await self.async_redis.zrandmember(
                self.active_workers_key, sample_size, withscores=True
            )

            if random_workers:
                worker_pairs = [(random_workers[i], random_workers[i+1]) for i in range(0, len(random_workers), 2)]
            else:
                worker_pairs = []

            for worker_id, zset_score in worker_pairs:
                worker_key = f"{self.redis_prefix}:{self.worker_prefix}:{worker_id}"
                hash_heartbeat = await self.async_redis.hget(worker_key, 'last_heartbeat')

                if not hash_heartbeat:
                    await self.async_redis.zrem(self.active_workers_key, worker_id)
                    logger.debug(f"Partial check: removed {worker_id}")
                else:
                    hash_time = float(hash_heartbeat)
                    zset_score_float = float(zset_score)
                    if abs(hash_time - zset_score_float) > 1.0:
                        await self.async_redis.zadd(self.active_workers_key, {worker_id: hash_time})
                        logger.debug(f"Partial check: synced {worker_id}")

        except Exception as e:
            import traceback
            traceback.print_exc()
            logger.error(f"Partial check error: {e}", exc_info=True)

    async def get_active_count(self) -> int:
        return await self.async_redis.zcard(self.active_workers_key)


    async def start_recovery(self, recovery_interval: float = 30.0):
        logger.debug(f"[Recovery] Starting offline worker recovery loop (interval={recovery_interval}s)")
        self._stop_recovery = False

        while not self._stop_recovery:
            try:
                await self._process_offline_workers_once()

                await asyncio.sleep(recovery_interval)

            except asyncio.CancelledError:
                logger.debug("[Recovery] Recovery loop cancelled")
                break
            except Exception as e:
                logger.error(f"[Recovery Loop] Unexpected error: {e}")
                await asyncio.sleep(recovery_interval)

        logger.debug("[Recovery] Offline worker recovery loop stopped")

    async def start(self, recovery_interval: float = 30.0):
        await self.start_recovery(recovery_interval)

    async def _process_offline_workers_once(self):
        

        async for worker_key, worker_data in self.find_all_offline_workers():
            
            if self._stop_recovery:
                break

            worker_id = worker_key.split(':')[-1]

            lock_key = f"{self.redis_prefix}:RECOVERY:WORKER_LOCK:{worker_id}"
            worker_lock = AsyncLock(
                self.async_redis,
                lock_key,
                timeout=300,  
                blocking=False  
            )

            if not await worker_lock.acquire():
                logger.debug(
                    f"[Recovery] Worker {worker_id} is being processed by another worker, skipping"
                )
                continue

            try:
                logger.debug(f"[Recovery] Processing worker={worker_id}")

                for key, value in worker_data.items():
                    if not key.startswith('group_info:'):
                        continue

                    try:
                        group_info = json.loads(value)
                        task_name = group_info.get('task_name')

                        already_transferred = group_info.get('messages_transferred', 'false')
                        if already_transferred == 'true':
                            logger.debug(
                                f"[Recovery] Group_info already transferred: worker={worker_id}, "
                                f"task={task_name}, skipping"
                            )
                            continue

                        if task_name not in self.tasks:
                            logger.debug(f"[Recovery] Task {task_name} not in managed tasks, skipping {self.tasks=}")
                            continue

                        task_event_queue = self.task_event_queues.get(task_name)
                        if not task_event_queue:
                            logger.debug(f"[Recovery] No event queue for task {task_name}, skipping")
                            continue

                        queue_name = group_info.get('queue')  
                        group_name = group_info.get('group_name')
                        offline_consumer_name = group_info.get('consumer_name')
                        stream_key_suffix = group_info.get('stream_key')  

                        if not all([queue_name, group_name, offline_consumer_name]):
                            logger.debug(f"[Recovery] Incomplete group_info: {group_info}")
                            continue

                        stream_key = stream_key_suffix

                        logger.debug(
                            f"[Recovery] Processing worker={worker_id}, task={task_name}, "
                            f"queue={queue_name}, group={group_name}"
                        )

                        success = await self._transfer_messages(
                            stream_key=stream_key,
                            group_name=group_name,
                            offline_consumer_name=offline_consumer_name,
                            task_name=task_name,
                            queue_name=queue_name,
                            task_event_queue=task_event_queue,
                            worker_id=worker_id,
                            worker_data=worker_data,
                            group_info_key=key
                        )

                        if success:
                            logger.debug(
                                f"[Recovery] Successfully transferred messages for "
                                f"worker={worker_id}, task={task_name}, queue={queue_name}"
                            )
                        else:
                            logger.debug(
                                f"[Recovery] Failed to transfer messages for "
                                f"worker={worker_id}, task={task_name}, queue={queue_name}"
                            )

                    except json.JSONDecodeError as e:
                        logger.error(f"[Recovery] Failed to parse group_info: {e}")
                    except Exception as e:
                        logger.error(f"[Recovery] Error processing group_info: {e}")

                logger.debug(f"[Recovery] Checking all group_info statuses for worker={worker_id}")

                fresh_worker_data = await self.get_worker_info(worker_id)
                if not fresh_worker_data:
                    logger.debug(f"[Recovery] Worker {worker_id} not found in Redis, skipping final check")
                    continue

                all_group_infos_transferred = True
                group_info_count = 0
                transferred_count = 0

                for key, value in fresh_worker_data.items():
                    if not key.startswith('group_info:'):
                        continue

                    group_info_count += 1
                    try:
                        group_info = json.loads(value)
                        is_transferred = group_info.get('messages_transferred', 'false')

                        if is_transferred == 'true':
                            transferred_count += 1
                            logger.debug(
                                f"[Recovery] Group_info {key} already transferred for worker={worker_id}"
                            )
                        else:
                            all_group_infos_transferred = False
                            logger.debug(
                                f"[Recovery] Group_info {key} NOT yet transferred for worker={worker_id}, "
                                f"cannot mark worker-level messages_transferred=true"
                            )
                    except json.JSONDecodeError as e:
                        logger.error(f"[Recovery] Failed to parse group_info {key}: {e}")
                        all_group_infos_transferred = False

                logger.debug(
                    f"[Recovery] Worker {worker_id} has {group_info_count} group_infos, "
                    f"{transferred_count} transferred"
                )

                if group_info_count == 0:
                    await self.async_redis.hset(
                        worker_key,
                        'messages_transferred',
                        'true'
                    )
                    logger.debug(
                        f"[Recovery] Worker {worker_id} has no group_infos, "
                        f"marked worker-level messages_transferred=true"
                    )
                elif all_group_infos_transferred:
                    await self.async_redis.hset(
                        worker_key,
                        'messages_transferred',
                        'true'
                    )
                    logger.debug(
                        f"[Recovery] All {group_info_count} group_infos transferred successfully for worker={worker_id}, "
                        f"marked worker-level messages_transferred=true"
                    )
                else:
                    logger.debug(
                        f"[Recovery] Not all group_infos transferred for worker={worker_id} "
                        f"({transferred_count}/{group_info_count}), will retry in next cycle"
                    )

            except Exception as e:
                logger.error(f"[Recovery] Error processing worker {worker_key}: {e}")
            finally:
                await worker_lock.release()
                logger.debug(f"[Recovery] Released lock for worker={worker_id}")

    async def _transfer_messages(
        self,
        stream_key: str,
        group_name: str,
        offline_consumer_name: str,
        task_name: str,
        queue_name: str,
        task_event_queue: asyncio.Queue,
        worker_id: str,
        worker_data: dict,
        group_info_key: str
    ) -> bool:

        total_transferred = 0

        try:
            messages_transferred = worker_data.get('messages_transferred', 'false')
            if isinstance(messages_transferred, bytes):
                messages_transferred = messages_transferred.decode('utf-8')

            if messages_transferred.lower() == 'true':
                logger.debug(f"[Recovery] Messages already transferred for worker={worker_id}")
                return False

            try:
                batch_size = 100

                while True:
                    detailed_pending = await self.async_redis.xpending_range(
                        stream_key,
                        group_name,
                        min='-',
                        max='+',
                        count=batch_size,
                        consumername=offline_consumer_name
                    )

                    if not detailed_pending:
                        logger.debug(
                            f"[Recovery] No pending messages for worker={worker_id}, "
                            f"task={task_name}, queue={queue_name}"
                        )
                        break

                    logger.debug(
                        f"[Recovery] Found {len(detailed_pending)} pending messages for "
                        f"worker={worker_id}, task={task_name}, queue={queue_name}"
                    )

                    message_ids = [msg['message_id'] for msg in detailed_pending]

                    claimed_messages = await self.async_redis.xclaim(
                        stream_key,
                        group_name,
                        self.worker_id,  
                        min_idle_time=0,
                        message_ids=message_ids
                    )

                    if not claimed_messages:
                        logger.debug(
                            f"[Recovery] Failed to claim messages for worker={worker_id}, "
                            f"task={task_name}, queue={queue_name}"
                        )
                        break

                    logger.debug(
                        f"[Recovery] Claimed {len(claimed_messages)} messages for "
                        f"worker={worker_id}, task={task_name}, queue={queue_name}"
                    )

                    for msg_id, msg_data in claimed_messages:
                        try:
                            if isinstance(msg_id, bytes):
                                msg_id = msg_id.decode('utf-8')

                            data_field = msg_data.get(b'data') or msg_data.get('data')
                            if not data_field:
                                logger.debug(f"[Recovery] No data field in message {msg_id}")
                                continue

                            parsed_data = msgpack.unpackb(data_field, raw=False)

                            parsed_data['_task_name'] = task_name
                            parsed_data['queue'] = queue_name

                            task_item = {
                                'queue': queue_name,
                                'event_id': msg_id,
                                'event_data': parsed_data,
                                'consumer': self.worker_id,
                                'group_name': group_name
                            }

                            await task_event_queue.put(task_item)
                            total_transferred += 1

                            logger.debug(
                                f"[Recovery] Put message {msg_id} into event_queue for task={task_name}"
                            )

                        except Exception as e:
                            logger.error(f"[Recovery] Error processing message {msg_id}: {e}")

            except Exception as e:
                logger.error(
                    f"[Recovery] Error during message recovery for worker={worker_id}, "
                    f"task={task_name}, queue={queue_name}: {e}, will still mark as transferred"
                )


            group_info_value = await self.async_redis.hget(
                self._get_worker_key(worker_id),
                group_info_key
            )

            if group_info_value:
                if isinstance(group_info_value, bytes):
                    group_info_value = group_info_value.decode('utf-8')

                group_info = json.loads(group_info_value)
                group_info['messages_transferred'] = 'true'

                await self.update_worker_field(
                    worker_id,
                    group_info_key,
                    json.dumps(group_info)
                )
                logger.debug(
                    f"[Recovery] Marked group_info transferred: worker={worker_id}, "
                    f"task={task_name}, queue={queue_name}, key={group_info_key}, transferred={total_transferred} messages"
                )
            else:
                logger.debug(
                    f"[Recovery] group_info not found in Redis for key={group_info_key}, "
                    f"worker={worker_id}, cannot mark as transferred"
                )

          

            return True  

        except Exception as e:
            logger.error(
                f"[Recovery] Error transferring messages for worker={worker_id}, "
                f"task={task_name}, queue={queue_name}: {e}"
            )
            return False  

    def stop_recovery(self):
        self._stop_recovery = True


    async def start_cleanup_task(self, cleanup_interval: float = 86400.0):
        logger.debug(f"[Cleanup] Starting offline worker cleanup task (interval={cleanup_interval}s)")
        self._stop_cleanup = False

        while not self._stop_cleanup:
            try:
                await self._cleanup_offline_workers_once()

                await asyncio.sleep(cleanup_interval)

            except asyncio.CancelledError:
                logger.debug("[Cleanup] Cleanup task cancelled")
                break
            except Exception as e:
                logger.error(f"[Cleanup] Unexpected error: {e}", exc_info=True)
                await asyncio.sleep(cleanup_interval)

        logger.debug("[Cleanup] Offline worker cleanup task stopped")

    async def _cleanup_offline_workers_once(self):
        current_time = time.time()
        one_day_ago = current_time - 86400  
        cleanup_count = 0

        try:
            all_worker_ids = await self.get_all_workers()
            logger.debug(f"[Cleanup] Scanning {len(all_worker_ids)} workers for cleanup")

            for worker_id in all_worker_ids:
                if self._stop_cleanup:
                    break

                if isinstance(worker_id, bytes):
                    worker_id = worker_id.decode('utf-8')

                lock_key = f"{self.redis_prefix}:CLEANUP:WORKER_LOCK:{worker_id}"
                worker_lock = AsyncLock(
                    self.async_redis,
                    lock_key,
                    timeout=60,  
                    blocking=False  
                )

                if not await worker_lock.acquire():
                    logger.debug(f"[Cleanup] Worker {worker_id} is being processed by another worker, skipping")
                    continue

                try:
                    worker_info = await self.get_worker_info(worker_id)

                    if not worker_info:
                        logger.debug(
                            f"[Cleanup] Worker {worker_id} in registry but hash not found, "
                            f"cleaning up inconsistent data"
                        )
                        await self.delete_worker(worker_id)
                        cleanup_count += 1
                        continue

                    is_alive = worker_info.get('is_alive', 'false')
                    messages_transferred = worker_info.get('messages_transferred', 'false')
                    offline_time_str = worker_info.get('offline_time', '0')

                    is_alive_bool = (is_alive == 'true')
                    messages_transferred_bool = (messages_transferred == 'true')

                    try:
                        offline_time = float(offline_time_str)
                    except (ValueError, TypeError):
                        offline_time = 0

                    should_cleanup = (
                        not is_alive_bool  
                        and messages_transferred_bool  
                        and offline_time > 0  
                        and offline_time < one_day_ago  
                    )

                    if should_cleanup:
                        offline_duration = current_time - offline_time
                        offline_hours = offline_duration / 3600

                        logger.debug(
                            f"[Cleanup] Cleaning up worker {worker_id}: "
                            f"offline for {offline_hours:.1f} hours, "
                            f"messages_transferred={messages_transferred}"
                        )

                        await self.delete_worker(worker_id)

                        cleanup_count += 1
                        logger.debug(f"[Cleanup] Successfully cleaned up worker {worker_id}")

                    else:
                        if is_alive_bool:
                            reason = "still alive"
                        elif not messages_transferred_bool:
                            reason = "messages not transferred"
                        elif offline_time == 0:
                            reason = "no offline_time"
                        elif offline_time >= one_day_ago:
                            offline_duration = current_time - offline_time
                            offline_hours = offline_duration / 3600
                            reason = f"offline for only {offline_hours:.1f} hours (< 24h)"
                        else:
                            reason = "unknown"

                        logger.debug(
                            f"[Cleanup] Worker {worker_id} not eligible for cleanup: {reason}"
                        )

                except Exception as e:
                    logger.error(f"[Cleanup] Error processing worker {worker_id}: {e}", exc_info=True)
                finally:
                    await worker_lock.release()

            if cleanup_count > 0:
                logger.debug(f"[Cleanup] Cleaned up {cleanup_count} offline workers")
            else:
                logger.debug("[Cleanup] No workers eligible for cleanup")

        except Exception as e:
            logger.error(f"[Cleanup] Error during cleanup: {e}", exc_info=True)

    def stop_cleanup(self):
        self._stop_cleanup = True


__all__ = [
    'WorkerManager',
]



import logging
import os
import asyncio
from contextlib import asynccontextmanager
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from jettask.webui.config import webui_config
from jettask.db.connector import get_pg_engine_and_factory
import uvicorn

logger = logging.getLogger(__name__)

CONFIG_SYNC_INTERVAL = int(os.environ.get('JETTASK_CONFIG_SYNC_INTERVAL', 300))


async def _config_sync_loop(app: FastAPI, interval: int):

    await asyncio.sleep(5)

    await _sync_all_configs(app)

    while True:
        try:
            await asyncio.sleep(interval)
            await _sync_all_configs(app)
        except asyncio.CancelledError:
            logger.info("配置同步任务被取消")
            break
        except Exception as e:
            logger.error(f"配置同步任务异常: {e}", exc_info=True)
            await asyncio.sleep(30)


async def _sync_all_configs(app: FastAPI):
    from jettask.webui.services.config_service import ConfigService
    from jettask.db.models.config import Config

    try:
        if not hasattr(app.state, 'meta_db_session_factory'):
            logger.warning("元数据库会话工厂未初始化，跳过配置同步")
            return

        if not hasattr(app.state, 'namespace_manager'):
            logger.warning("命名空间管理器未初始化，跳过配置同步")
            return

        meta_db_session_factory = app.state.meta_db_session_factory
        namespace_manager = app.state.namespace_manager

        from jettask.config import DynamicConfig

        namespaces = await namespace_manager.list_namespaces(enabled_only=True)

        for ns in namespaces:
            try:
                redis_client = await ns.get_redis_client()
                prefix = ns.redis_prefix

                async with meta_db_session_factory() as pg_session:
                    task_configs = await ConfigService.get_all_task_configs_from_pg(pg_session, ns.name)

                    for scope, configs in task_configs.items():
                        parsed = Config.parse_task_scope(scope)
                        if not parsed:
                            continue

                        task_name = parsed['task_name']
                        queue = parsed['queue']

                        pg_task_version = await ConfigService.get_task_version_from_pg(
                            pg_session, ns.name, queue, task_name
                        )

                        config_key = DynamicConfig.get_task_config_key(prefix, task_name)
                        redis_task_version = await redis_client.hget(config_key, '_version')
                        redis_task_version = int(redis_task_version) if redis_task_version else 0

                        logger.debug(
                            f"[CONFIG_SYNC] {ns.name}:{task_name}: "
                            f"PG版本={pg_task_version}, Redis版本={redis_task_version}"
                        )

                        if redis_task_version > 0 and redis_task_version >= pg_task_version:
                            continue
                        if redis_task_version == 0 and pg_task_version == 0:
                            logger.debug(f"[CONFIG_SYNC] {ns.name}:{task_name}: 两边版本号都为0，跳过")
                            continue

                        logger.info(
                            f"[CONFIG_SYNC] 恢复任务配置: {ns.name}:{task_name} "
                            f"(PG={pg_task_version}, Redis={redis_task_version})"
                        )
                        configs_with_version = {**configs, '_version': str(pg_task_version)}
                        await ConfigService._write_flat_config_to_redis(
                            redis_client, prefix, task_name, configs_with_version
                        )

                    system_configs = await ConfigService.get_all_system_configs_from_pg(pg_session)
                    system_config_key = DynamicConfig.get_system_config_key(prefix)

                    for key, value in system_configs.items():
                        exists = await redis_client.hexists(system_config_key, key)
                        if not exists:
                            logger.info(f"[CONFIG_SYNC] 恢复系统配置: {key}")
                            await DynamicConfig.set_system_config_to_redis(
                                redis_client, prefix, key, value,
                                publish_change=False
                            )

            except Exception as e:
                logger.error(f"同步配置到命名空间 {ns.name} 失败: {e}", exc_info=True)

    except Exception as e:
        logger.error(f"配置同步失败: {e}", exc_info=True)


@asynccontextmanager
async def lifespan(app: FastAPI):

    try:
        log_format = os.environ.get('JETTASK_LOG_FORMAT', 'text').lower()
        log_level = logging.INFO

        from jettask.utils.task_logger import JSONFormatter, ExtendedTextFormatter, TaskContextFilter
        import sys

        jettask_logger_names = [name for name in logging.Logger.manager.loggerDict
                                if name.startswith('jettask')]

        if not jettask_logger_names:
            jettask_logger_names = ['jettask']

        for logger_name in jettask_logger_names:
            jettask_logger = logging.getLogger(logger_name)
            jettask_logger.setLevel(log_level)

            jettask_logger.handlers.clear()

            handler = logging.StreamHandler(sys.stderr)
            handler.addFilter(TaskContextFilter())

            if log_format == 'json':
                handler.setFormatter(JSONFormatter())
            else:
                handler.setFormatter(ExtendedTextFormatter(
                    '%(asctime)s - %(levelname)s - [%(task_id)s] - %(name)s - %(message)s'
                ))

            jettask_logger.addHandler(handler)
            jettask_logger.propagate = False  

        if log_format == 'json':
            logger.info("日志格式已配置为 JSON (仅jettask相关日志)")
        else:
            logger.info("日志格式已配置为 TEXT (仅jettask相关日志)")

        db_info = webui_config.get_database_info()

        logger.info("=" * 60)
        logger.info("任务中心配置:")
        logger.info(f"  配置模式: {'Nacos' if webui_config.use_nacos else '环境变量'}")

        if db_info['host']:
            logger.info(f"  元数据库: {db_info['host']}:{db_info['port']}/{db_info['database']}")
        else:
            logger.info(f"  元数据库: {webui_config.pg_url}")

        logger.info(f"  Redis: {webui_config._mask_url(webui_config.redis_url)}")
        logger.info(f"  Redis Prefix: {webui_config.redis_prefix}")
        logger.info(f"  API服务: {webui_config.api_host}:{webui_config.api_port}")
        logger.info(f"  基础URL: {webui_config.base_url}")
        logger.info("=" * 60)

        meta_db_engine, meta_db_session_factory = get_pg_engine_and_factory(
            webui_config.meta_database_url
        )
        app.state.meta_db_engine = meta_db_engine
        app.state.meta_db_session_factory = meta_db_session_factory
        logger.info("元数据库会话工厂已初始化")

        from jettask.core.namespace import NamespaceManagerDB
        namespace_manager = NamespaceManagerDB(auto_refresh=True, refresh_interval=60)
        app.state.namespace_manager = namespace_manager
        logger.info("命名空间管理器已初始化")


        sync_task = asyncio.create_task(
            _config_sync_loop(app, CONFIG_SYNC_INTERVAL)
        )
        app.state.config_sync_task = sync_task
        logger.info(f"配置同步后台任务已启动（间隔 {CONFIG_SYNC_INTERVAL} 秒）")

        logger.info("JetTask WebUI 启动成功")
    except Exception as e:
        logger.error(f"启动失败: {e}")
        import traceback
        traceback.print_exc()
        raise

    yield

    if hasattr(app.state, 'config_sync_task') and app.state.config_sync_task:
        app.state.config_sync_task.cancel()
        try:
            await app.state.config_sync_task
        except asyncio.CancelledError:
            pass
        logger.info("配置同步后台任务已停止")

    if hasattr(app.state, 'namespace_manager') and app.state.namespace_manager:
        await app.state.namespace_manager.close()
        logger.info("命名空间管理器已关闭")

    if hasattr(app.state, 'meta_db_engine') and app.state.meta_db_engine:
        await app.state.meta_db_engine.dispose()
        logger.info("元数据库连接已关闭")


app = FastAPI(title="Jettask Monitor", lifespan=lifespan)

from jettask.webui.middleware import UnifiedAuthMiddleware
from jettask.webui.utils.jwt_utils import JWTManager

api_key = os.environ.get('JETTASK_API_KEY')
remote_token_verify_url = webui_config.remote_token_verify_url

jwt_manager = None
if remote_token_verify_url:
    jwt_manager = JWTManager(
        secret_key=webui_config.jwt_secret_key,
        algorithm=webui_config.jwt_algorithm,
        access_token_expire_minutes=webui_config.jwt_access_token_expire_minutes,
        refresh_token_expire_days=webui_config.jwt_refresh_token_expire_days
    )
    logger.info(f"JWT Manager 已初始化（用于前端用户认证）")

app.add_middleware(
    UnifiedAuthMiddleware,
    api_key=api_key,
    jwt_manager=jwt_manager
)

auth_methods = []
if api_key:
    auth_methods.append("API Key（后端服务）")
if jwt_manager:
    auth_methods.append("JWT Token（前端用户）")

if auth_methods:
    logger.info(f"UnifiedAuthMiddleware 已注册 - 支持鉴权方式: {', '.join(auth_methods)}")
else:
    logger.warning("UnifiedAuthMiddleware 已注册 - 未配置任何鉴权方式，所有请求无需认证")

from jettask.webui.middleware import NamespaceMiddleware
app.add_middleware(NamespaceMiddleware)
logger.info("NamespaceMiddleware 已注册 - 所有包含 {namespace} 的路由将自动注入命名空间上下文")

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  
    allow_credentials=True,
    allow_methods=["*"],  
    allow_headers=["*"],  
    max_age=3600,  
)
logger.info("CORSMiddleware 已注册（最先执行，确保所有响应都带 CORS headers）")

from jettask.webui.api import api_router
app.include_router(api_router)


if __name__ == "__main__":
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    )
    
    uvicorn.run(app, host="0.0.0.0", port=8000)


"""
QueueSnapshot 模型

队列快照表 - 定时采集队列积压数据和 Worker 数量统计
"""
from sqlalchemy import Column, Text, Integer, BigInteger, TIMESTAMP, Index
from datetime import datetime, timezone

from ..base import Base


class QueueSnapshot(Base):
    """
    队列快照表

    合并积压指标和 Worker 指标，定时采集：
    - 积压数据（backlog、pending_read、offset）
    - Worker 数量统计（online、offline、total）

    存储两类记录：
    - task 明细：每个 (queue, task_name) 一条，最细粒度
    - namespace 汇总：queue='', task_name=''，Worker 全局去重
    """
    __tablename__ = 'queue_snapshot'

    time_bucket = Column(
        TIMESTAMP(timezone=True),
        primary_key=True,
        comment='时间桶（分钟级别）'
    )
    namespace = Column(
        Text,
        primary_key=True,
        comment='命名空间'
    )
    queue = Column(
        Text,
        primary_key=True,
        comment='队列名称（namespace 汇总时为空字符串）'
    )
    task_name = Column(
        Text,
        primary_key=True,
        comment='任务名称（namespace 汇总时为空字符串）'
    )

    backlog = Column(
        Integer,
        nullable=False,
        default=0,
        comment='积压数量（已发送未ACK）= send_offset - ack_offset'
    )
    pending_read = Column(
        Integer,
        nullable=False,
        default=0,
        comment='待读取数量（已发送未读取）= send_offset - read_offset'
    )
    send_offset = Column(
        BigInteger,
        nullable=True,
        default=0,
        comment='发送 offset'
    )
    ack_offset = Column(
        BigInteger,
        nullable=True,
        default=0,
        comment='确认 offset'
    )
    read_offset = Column(
        BigInteger,
        nullable=True,
        default=0,
        comment='读取 offset'
    )

    online_count = Column(
        Integer,
        nullable=False,
        default=0,
        comment='在线 Worker 数量'
    )
    offline_count = Column(
        Integer,
        nullable=False,
        default=0,
        comment='离线 Worker 数量'
    )
    total_count = Column(
        Integer,
        nullable=False,
        default=0,
        comment='总 Worker 数量'
    )

    created_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        default=lambda: datetime.now(timezone.utc),
        comment='记录创建时间'
    )

    __table_args__ = (
        Index('idx_queue_snapshot_time', 'time_bucket'),
        Index('idx_queue_snapshot_ns_queue', 'namespace', 'queue'),
        Index('idx_queue_snapshot_ns_queue_task', 'namespace', 'queue', 'task_name'),
    )

    def to_dict(self):
        return {
            'time_bucket': self.time_bucket.isoformat() if self.time_bucket else None,
            'namespace': self.namespace,
            'queue': self.queue,
            'task_name': self.task_name,
            'backlog': self.backlog,
            'pending_read': self.pending_read,
            'send_offset': self.send_offset,
            'ack_offset': self.ack_offset,
            'read_offset': self.read_offset,
            'online_count': self.online_count,
            'offline_count': self.offline_count,
            'total_count': self.total_count,
            'created_at': self.created_at.isoformat() if self.created_at else None,
        }

    def __repr__(self):
        return (
            f"<QueueSnapshot("
            f"time_bucket={self.time_bucket}, "
            f"namespace={self.namespace}, "
            f"queue={self.queue}, "
            f"task={self.task_name}, "
            f"backlog={self.backlog}, "
            f"online={self.online_count}"
            f")>"
        )



"""
Jettask v2 CLI 主程序
"""

import sys


def main():
    """CLI 主入口"""
    print("Jettask v2 CLI")
    print("Usage: python -m jettask_v2.interfaces.cli.main [command]")
    sys.exit(1)


if __name__ == "__main__":
    main()



"""
Config 模型

对应 configs 表，用于存储动态配置（系统配置和任务配置）
"""
from sqlalchemy import Column, String, Text, Integer, TIMESTAMP, func, Index
from sqlalchemy.dialects.postgresql import JSONB
from typing import Optional, Dict, Any

from ..base import Base


class Config(Base):
    """
    动态配置表

    统一存储系统配置和任务配置：
    - system 类型：全局系统配置（如 DDSketch 精度、日志级别）
    - task 类型：任务级配置（如 rate_limit、max_retries）
    """
    __tablename__ = 'configs'

    id = Column(Integer, primary_key=True, autoincrement=True, comment='自增主键')

    config_type = Column(
        String(20),
        nullable=False,
        comment='配置类型：system（系统配置）| task（任务配置）'
    )

    scope = Column(
        String(500),
        nullable=True,
        comment='作用域，task 类型格式为 namespace:queue:task_name'
    )

    key = Column(
        String(100),
        nullable=False,
        comment='配置键名'
    )

    value = Column(
        JSONB,
        nullable=False,
        comment='配置值（JSONB 格式）'
    )

    description = Column(
        Text,
        nullable=True,
        comment='配置描述'
    )

    created_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        server_default=func.current_timestamp(),
        comment='创建时间'
    )
    updated_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        server_default=func.current_timestamp(),
        onupdate=func.current_timestamp(),
        comment='更新时间'
    )

    __table_args__ = (
        Index(
            'uq_configs_type_scope_key',
            'config_type', 'scope', 'key',
            unique=True
        ),
        Index('idx_configs_type', 'config_type'),
        Index('idx_configs_scope', 'scope'),
        Index('idx_configs_updated_at', 'updated_at'),
    )

    def to_dict(self) -> Dict[str, Any]:
        return {
            'id': self.id,
            'config_type': self.config_type,
            'scope': self.scope,
            'key': self.key,
            'value': self.value,
            'description': self.description,
            'created_at': self.created_at.isoformat() if self.created_at else None,
            'updated_at': self.updated_at.isoformat() if self.updated_at else None,
        }

    @classmethod
    def make_task_scope(cls, namespace: str, queue: str, task_name: str) -> str:
        return f"{namespace}:{queue}:{task_name}"

    @classmethod
    def parse_task_scope(cls, scope: str) -> Optional[Dict[str, str]]:
        if not scope:
            return None
        parts = scope.split(':', 2)
        if len(parts) != 3:
            return None
        return {
            'namespace': parts[0],
            'queue': parts[1],
            'task_name': parts[2],
        }

    def __repr__(self) -> str:
        return (
            f"<Config(id={self.id}, type='{self.config_type}', "
            f"scope='{self.scope}', key='{self.key}')>"
        )



"""
统一的 Redis Pub/Sub 管理器

提供统一的 Pub/Sub 订阅管理、错误处理和自动重连机制。

注意：
- 连接池级别的配置（socket_timeout, health_check_interval）应使用
  connector.py 的 get_async_redis_pool_for_pubsub()
- 本模块只处理应用层的订阅逻辑、消息回调和断线重连

使用示例:

    # 创建订阅
    subscription = await PubSubManager.subscribe(
        redis_client=redis,
        channel="my_channel",
        on_message=my_callback,
        name="my_listener"
    )

    # 停止订阅
    await subscription.stop()

    # 或使用上下文管理器
    async with PubSubManager.subscribe(...) as subscription:
        await asyncio.sleep(60)  # 监听 60 秒
"""

import asyncio
import json
import logging
from typing import Any, Callable, Optional, Union, Awaitable
from dataclasses import dataclass

import redis.asyncio as redis

logger = logging.getLogger(__name__)


@dataclass
class PubSubConfig:
    """Pub/Sub 配置"""
    initial_retry_delay: float = 1.0  
    max_retry_delay: float = 30.0  
    retry_backoff_multiplier: float = 2.0  

    mark_connection: bool = True


class PubSubSubscription:
    """
    Pub/Sub 订阅实例

    管理单个 channel 的订阅生命周期，包括：
    - 自动重连（应用层，当 listen() 抛异常时重新订阅）
    - 连接标记（防止被空闲连接清理）

    注意：连接池级别的健康检查由 Redis 连接池处理，
    参见 connector.py 的 get_async_redis_pool_for_pubsub()
    """

    def __init__(
        self,
        redis_client: redis.Redis,
        channel: str,
        on_message: Callable[[dict], Union[None, Awaitable[None]]],
        name: str = None,
        config: PubSubConfig = None,
        parse_json: bool = True,
    ):
        self._redis = redis_client
        self._channel = channel
        self._on_message = on_message
        self._name = name or channel
        self._config = config or PubSubConfig()
        self._parse_json = parse_json

        self._pubsub: Optional[redis.client.PubSub] = None
        self._running = False
        self._listener_task: Optional[asyncio.Task] = None

    @property
    def channel(self) -> str:
        return self._channel

    @property
    def is_running(self) -> bool:
        return self._running

    async def start(self):
        if self._running:
            logger.debug(f"[PubSub:{self._name}] Already running")
            return

        self._running = True
        self._pubsub = await self._create_and_subscribe()
        self._listener_task = asyncio.create_task(self._listen_loop())

        logger.debug(f"[PubSub:{self._name}] Started on channel: {self._channel}")

    async def stop(self):
        if not self._running:
            return

        self._running = False

        if self._listener_task:
            self._listener_task.cancel()
            try:
                await self._listener_task
            except asyncio.CancelledError:
                pass
            self._listener_task = None

        if self._pubsub:
            try:
                await self._pubsub.unsubscribe(self._channel)
                await self._pubsub.close()
            except Exception as e:
                logger.debug(f"[PubSub:{self._name}] Error closing pubsub: {e}")
            self._pubsub = None

        logger.debug(f"[PubSub:{self._name}] Stopped")

    async def _create_and_subscribe(self) -> redis.client.PubSub:
        if self._pubsub:
            try:
                await self._pubsub.close()
            except Exception:
                pass

        pubsub = self._redis.pubsub()
        await pubsub.subscribe(self._channel)

        if self._config.mark_connection:
            self._mark_pubsub_connection(pubsub)

        logger.debug(f"[PubSub:{self._name}] Subscribed to channel: {self._channel}")
        return pubsub

    def _mark_pubsub_connection(self, pubsub: redis.client.PubSub):
        if hasattr(pubsub, 'connection') and pubsub.connection:
            pubsub.connection._is_pubsub_connection = True
            logger.debug(
                f"[PubSub:{self._name}] Marked connection {id(pubsub.connection)} "
                "to prevent idle cleanup"
            )

    async def _listen_loop(self):
        retry_delay = self._config.initial_retry_delay

        while self._running:
            try:
                async for message in self._pubsub.listen():
                    if not self._running:
                        break

                    if message['type'] == 'message':
                        await self._handle_message(message)

                retry_delay = self._config.initial_retry_delay

            except asyncio.CancelledError:
                logger.debug(f"[PubSub:{self._name}] Listen loop cancelled")
                break

            except Exception as e:
                if not self._running:
                    break

                error_msg = str(e).lower()
                is_connection_error = any(
                    keyword in error_msg
                    for keyword in ['connection', 'closed', 'buffer', 'timeout', 'eof']
                )

                if is_connection_error:
                    logger.warning(
                        f"[PubSub:{self._name}] Connection lost: {e}, "
                        f"reconnecting in {retry_delay}s..."
                    )
                else:
                    logger.error(f"[PubSub:{self._name}] Error in listen loop: {e}")

                await asyncio.sleep(retry_delay)

                try:
                    self._pubsub = await self._create_and_subscribe()
                    logger.info(f"[PubSub:{self._name}] Reconnected successfully")
                    retry_delay = self._config.initial_retry_delay
                except Exception as reconnect_error:
                    logger.error(
                        f"[PubSub:{self._name}] Reconnect failed: {reconnect_error}"
                    )
                    retry_delay = min(
                        retry_delay * self._config.retry_backoff_multiplier,
                        self._config.max_retry_delay
                    )

        logger.debug(f"[PubSub:{self._name}] Listen loop exited")

    async def _handle_message(self, message: dict):
        try:
            data = message['data']

            if self._parse_json:
                try:
                    data = json.loads(data)
                except (json.JSONDecodeError, TypeError):
                    pass

            if asyncio.iscoroutinefunction(self._on_message):
                await self._on_message(data)
            else:
                self._on_message(data)

        except Exception as e:
            logger.error(f"[PubSub:{self._name}] Error handling message: {e}")

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.stop()


class PubSubManager:
    """
    Pub/Sub 管理器

    提供统一的订阅创建和管理接口
    """

    @staticmethod
    async def subscribe(
        redis_client: redis.Redis,
        channel: str,
        on_message: Callable[[dict], Union[None, Awaitable[None]]],
        name: str = None,
        config: PubSubConfig = None,
        parse_json: bool = True,
        auto_start: bool = True,
    ) -> PubSubSubscription:
        subscription = PubSubSubscription(
            redis_client=redis_client,
            channel=channel,
            on_message=on_message,
            name=name,
            config=config,
            parse_json=parse_json,
        )

        if auto_start:
            await subscription.start()

        return subscription

    @staticmethod
    async def publish(
        redis_client: redis.Redis,
        channel: str,
        message: Any,
        serialize_json: bool = True,
    ) -> int:
        if serialize_json and not isinstance(message, str):
            message = json.dumps(message)

        return await redis_client.publish(channel, message)


class MultiChannelSubscription:
    """
    多 Channel 订阅管理器

    用于同时订阅多个 channel，共享一个 PubSub 连接
    """

    def __init__(
        self,
        redis_client: redis.Redis,
        name: str = "multi",
        config: PubSubConfig = None,
        parse_json: bool = True,
    ):
        self._redis = redis_client
        self._name = name
        self._config = config or PubSubConfig()
        self._parse_json = parse_json

        self._pubsub: Optional[redis.client.PubSub] = None
        self._running = False
        self._listener_task: Optional[asyncio.Task] = None

        self._handlers: dict[str, Callable] = {}

    async def subscribe(
        self,
        channel: str,
        on_message: Callable[[dict], Union[None, Awaitable[None]]],
    ):
        self._handlers[channel] = on_message

        if self._pubsub:
            await self._pubsub.subscribe(channel)
            logger.debug(f"[PubSub:{self._name}] Added subscription: {channel}")

    async def unsubscribe(self, channel: str):
        if channel in self._handlers:
            del self._handlers[channel]

            if self._pubsub:
                await self._pubsub.unsubscribe(channel)
                logger.debug(f"[PubSub:{self._name}] Removed subscription: {channel}")

    async def start(self):
        if self._running:
            return

        self._running = True
        self._pubsub = await self._create_and_subscribe()
        self._listener_task = asyncio.create_task(self._listen_loop())

        logger.debug(f"[PubSub:{self._name}] Started with {len(self._handlers)} channels")

    async def stop(self):
        if not self._running:
            return

        self._running = False

        if self._listener_task:
            self._listener_task.cancel()
            try:
                await self._listener_task
            except asyncio.CancelledError:
                pass

        if self._pubsub:
            try:
                await self._pubsub.close()
            except Exception:
                pass

        logger.debug(f"[PubSub:{self._name}] Stopped")

    async def _create_and_subscribe(self) -> redis.client.PubSub:
        if self._pubsub:
            try:
                await self._pubsub.close()
            except Exception:
                pass

        pubsub = self._redis.pubsub()

        for channel in self._handlers.keys():
            await pubsub.subscribe(channel)

        if self._config.mark_connection and hasattr(pubsub, 'connection') and pubsub.connection:
            pubsub.connection._is_pubsub_connection = True

        return pubsub

    async def _listen_loop(self):
        retry_delay = self._config.initial_retry_delay

        while self._running:
            try:
                async for message in self._pubsub.listen():
                    if not self._running:
                        break

                    if message['type'] == 'message':
                        channel = message['channel']
                        if isinstance(channel, bytes):
                            channel = channel.decode('utf-8')

                        handler = self._handlers.get(channel)
                        if handler:
                            await self._handle_message(message, handler)

                retry_delay = self._config.initial_retry_delay

            except asyncio.CancelledError:
                break
            except Exception as e:
                if not self._running:
                    break

                logger.warning(f"[PubSub:{self._name}] Error: {e}, reconnecting in {retry_delay}s")
                await asyncio.sleep(retry_delay)

                try:
                    self._pubsub = await self._create_and_subscribe()
                    retry_delay = self._config.initial_retry_delay
                except Exception:
                    retry_delay = min(
                        retry_delay * self._config.retry_backoff_multiplier,
                        self._config.max_retry_delay
                    )

    async def _handle_message(self, message: dict, handler: Callable):
        try:
            data = message['data']
            if self._parse_json:
                try:
                    data = json.loads(data)
                except (json.JSONDecodeError, TypeError):
                    pass

            if asyncio.iscoroutinefunction(handler):
                await handler(data)
            else:
                handler(data)
        except Exception as e:
            logger.error(f"[PubSub:{self._name}] Error handling message: {e}")

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.stop()


__all__ = [
    'PubSubConfig',
    'PubSubSubscription',
    'PubSubManager',
    'MultiChannelSubscription',
]
